import pathlib
from setuptools import setup, find_packages, Extension

import numpy as np

from eiseg import __APPNAME__, __VERSION__

# from Cython.Build import cythonize

HERE = pathlib.Path(__file__).parent

README = (HERE / "README.md").read_text(encoding="utf-8")

with open("requirements.txt") as fin:
    REQUIRED_PACKAGES = fin.read()

ext_modules = [
    Extension(
        "pycocotools._mask",
        sources=[
            "./eiseg/util/coco/common/maskApi.c",
            "./eiseg/util/coco/pycocotools/_mask.pyx",
        ],
        include_dirs=[np.get_include(), "./eiseg/util/coco/common"],
        extra_compile_args=["-Wno-cpp", "-Wno-unused-function", "-std=c99"], )
]
packages = find_packages(exclude=("test", ))
setup(
    name=__APPNAME__,
    version=__VERSION__,
    description="交互式标注软件",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/PaddlePaddle/PaddleSeg",
    author="PaddleSeg & PaddleCV-SIG",
    author_email="haoyuying@baidu.com",
    license="Apache Software License",
    classifiers=[
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
    ],
    packages=find_packages(exclude=("test", )),

    # packages=["EISeg"],
    include_package_data=True,
    install_requires=REQUIRED_PACKAGES,
    entry_points={"console_scripts": ["eiseg=eiseg.run:main", ]}, )



# Copyright (c) 2021 PaddlePaddle Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
This code is based on https://github.com/niecongchong/RS-building-regularization
Ths copyright of niecongchong/RS-building-regularization is as follows:
Apache License [see LICENSE for details]
"""

import math


# 顺时针旋转
def Nrotation_angle_get_coor_coordinates(point, center, angle):
    src_x, src_y = point
    center_x, center_y = center
    radian = math.radians(angle)
    dest_x = (src_x - center_x) * math.cos(radian) + \
             (src_y - center_y) * math.sin(radian) + center_x
    dest_y = (src_y - center_y) * math.cos(radian) - \
             (src_x - center_x) * math.sin(radian) + center_y
    # return (int(dest_x), int(dest_y))
    return (dest_x, dest_y)


# 逆时针旋转
def Srotation_angle_get_coor_coordinates(point, center, angle):
    src_x, src_y = point
    center_x, center_y = center
    radian = math.radians(angle)
    dest_x = (src_x - center_x) * math.cos(radian) - \
             (src_y - center_y) * math.sin(radian) + center_x
    dest_y = (src_x - center_x) * math.sin(radian) + \
             (src_y - center_y) * math.cos(radian) + center_y
    # return [int(dest_x), int(dest_y)]
    return (dest_x, dest_y)



# Copyright (c) 2021 PaddlePaddle Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
This code is based on https://github.com/niecongchong/RS-building-regularization
Ths copyright of niecongchong/RS-building-regularization is as follows:
Apache License [see LICENSE for details]
"""
"""
rdp
~~~
Pure Python implementation of the Ramer-Douglas-Peucker algorithm.
:copyright: (c) 2014 Fabian Hirschmann <fabian@hirschmann.email>
:license: MIT, see LICENSE.txt for more details.
"""

import numpy as np


def pldist(x0, x1, x2):
    """
    Calculates the distance from the point ``x0`` to the line given
    by the points ``x1`` and ``x2``.
    :param x0: a point
    :type x0: a 2x1 numpy array
    :param x1: a point of the line
    :type x1: 2x1 numpy array
    :param x2: another point of the line
    :type x2: 2x1 numpy array
    """
    x0, x1, x2 = x0[:2], x1[:2], x2[:2]  # discard timestamp
    if x1[0] == x2[0]:
        return np.abs(x0[0] - x1[0])
    return np.divide(
        np.linalg.norm(np.linalg.det([x2 - x1, x1 - x0])),
        np.linalg.norm(x2 - x1))


def _rdp(M, epsilon, dist):
    """
    Simplifies a given array of points.
    :param M: an array
    :type M: Nx2 numpy array
    :param epsilon: epsilon in the rdp algorithm
    :type epsilon: float
    :param dist: distance function
    :type dist: function with signature ``f(x1, x2, x3)``
    """
    dmax = 0.0
    index = -1
    for i in range(1, M.shape[0]):
        d = dist(M[i], M[0], M[-1])
        if d > dmax:
            index = i
            dmax = d
    if dmax > epsilon:
        r1 = _rdp(M[:index + 1], epsilon, dist)
        r2 = _rdp(M[index:], epsilon, dist)
        return np.vstack((r1[:-1], r2))
    else:
        return np.vstack((M[0], M[-1]))


def _rdp_nn(seq, epsilon, dist):
    """
    Simplifies a given array of points.
    :param seq: a series of points
    :type seq: sequence of 2-tuples
    :param epsilon: epsilon in the rdp algorithm
    :type epsilon: float
    :param dist: distance function
    :type dist: function with signature ``f(x1, x2, x3)``
    """
    return _rdp(np.array(seq), epsilon, dist).tolist()


def rdp(M, epsilon=0, dist=pldist):
    """
    Simplifies a given array of points.
    :param M: a series of points
    :type M: either a Nx2 numpy array or sequence of 2-tuples
    :param epsilon: epsilon in the rdp algorithm
    :type epsilon: float
    :param dist: distance function
    :type dist: function with signature ``f(x1, x2, x3)``
    """
    if "numpy" in str(type(M)):
        return _rdp(M, epsilon, dist)
    return _rdp_nn(M, epsilon, dist)



# Copyright (c) 2021 PaddlePaddle Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os.path as osp
import random

from eiseg import pjpath


class ColorMap(object):
    def __init__(self, color_path, shuffle=False):
        self.colors = []
        self.index = 0
        self.usedColors = []
        with open(color_path, "r") as f:
            colors = f.readlines()
        if shuffle:
            random.shuffle(colors)
        self.colors = [[int(x) for x in c.strip().split(",")] for c in colors]

    def get_color(self):
        color = self.colors[self.index]
        self.index = (self.index + 1) % len(self)
        return color

    def __len__(self):
        return len(self.colors)


colorMap = ColorMap(osp.join(pjpath, "config/colormap.txt"))



# Copyright (c) 2021 PaddlePaddle Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os.path as osp
from functools import partial

from qtpy import QtCore, QtGui, QtWidgets

from qtpy.QtGui import QIcon
from qtpy.QtCore import QTimer

from eiseg import pjpath, __APPNAME__, __VERSION__, logger
from eiseg.widget.create import creat_dock, create_button, create_slider, create_text
from widget import AnnotationScene, AnnotationView
from widget.create import *
from widget.table import TableWidget
from widget.vtk import VTKWidget

# log = logging.getLogger(__name__ + ".ui")


class Ui_EISeg(object):
    def __init__(self):
        super(Ui_EISeg, self).__init__()
        self.tr = partial(QtCore.QCoreApplication.translate, "APP_EISeg")

    def addVideoslider(self, MainWindow, frames):
        ## 时间轴设置
        widget = QtWidgets.QWidget()
        verticalLayout = QtWidgets.QVBoxLayout(widget)
        self.VideoDock = self.p_create_dock("VideoDock", self.tr("时间轴"), widget)
        VideoRegion = QtWidgets.QHBoxLayout()
        VideoRegion.setObjectName("VideoRegion")
        self.videoPlay = self.p_create_button(
            "videoPlay",
            self.tr("播放"),
            osp.join(pjpath, "resource/Play.png"),
            "", )
        VideoRegion.addWidget(self.videoPlay)
        self.sldTime, self.textTime, _ = self.p_create_slider(
            "sldTime", "textTime", "", 0, frames, 0, 1, True)
        VideoRegion.addWidget(
            create_text(self.CentralWidget, None, self.tr("帧数：")))
        VideoRegion.addWidget(self.textTime)
        VideoRegion.addWidget(self.sldTime)
        verticalLayout.addLayout(VideoRegion)
        self.VideoDock.setAllowedAreas(QtCore.Qt.BottomDockWidgetArea)
        MainWindow.addDockWidget(QtCore.Qt.BottomDockWidgetArea, self.VideoDock)

        # timer
        self.timer = QTimer()
        self.timer.setSingleShot(False)

    def setupUi(self, MainWindow):
        ## -- 主窗体设置 --
        MainWindow.setObjectName("MainWindow")
        MainWindow.setMinimumSize(QtCore.QSize(1366, 768))  # 1366x768的屏幕显示不全
        MainWindow.setWindowTitle(__APPNAME__ + " " + __VERSION__)
        MainWindow.setWindowIcon(QIcon())  # TODO: 默认图标需要换一个吗，貌似不能不显示图标
        self.CentralWidget = QtWidgets.QWidget(MainWindow)
        self.CentralWidget.setObjectName("CentralWidget")
        MainWindow.setCentralWidget(self.CentralWidget)
        ## -----
        ## -- 工具栏 --
        toolBar = QtWidgets.QToolBar(self)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred,
                                           QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(toolBar.sizePolicy().hasHeightForWidth())
        toolBar.setSizePolicy(sizePolicy)
        toolBar.setMinimumSize(QtCore.QSize(0, 33))
        toolBar.setMovable(True)
        toolBar.setAllowedAreas(QtCore.Qt.BottomToolBarArea |
                                QtCore.Qt.TopToolBarArea)
        toolBar.setObjectName("toolBar")
        self.toolBar = toolBar
        MainWindow.addToolBar(QtCore.Qt.TopToolBarArea, self.toolBar)
        ## -----
        ## -- 状态栏 --
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        self.statusbar.setStyleSheet("QStatusBar::item {border: none;}")
        MainWindow.setStatusBar(self.statusbar)
        self.statusbar.addPermanentWidget(
            self.show_logo(osp.join(pjpath, "resource/Paddle.png")))
        ## -----
        ## -- 图形区域 --
        ImageRegion = QtWidgets.QHBoxLayout(self.CentralWidget)
        ImageRegion.setObjectName("ImageRegion")
        # 滑动区域
        self.scrollArea = QtWidgets.QScrollArea(self.CentralWidget)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        ImageRegion.addWidget(self.scrollArea)
        # 图形显示
        self.scene = AnnotationScene()
        self.scene.addPixmap(QtGui.QPixmap())
        self.canvas = AnnotationView(self.scene, self)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding,
                                           QtWidgets.QSizePolicy.Expanding)
        self.canvas.setSizePolicy(sizePolicy)
        self.canvas.setAlignment(QtCore.Qt.AlignCenter)
        self.canvas.setAutoFillBackground(False)
        self.canvas.setStyleSheet("background-color: White")
        self.canvas.setObjectName("canvas")
        self.scrollArea.setWidget(self.canvas)
        ## -----
        ## -- 工作区 --
        self.p_create_dock = partial(self.creat_dock, MainWindow)
        self.p_create_button = partial(self.create_button, self.CentralWidget)
        # 模型加载
        widget = QtWidgets.QWidget()
        horizontalLayout = QtWidgets.QHBoxLayout(widget)
        self.ModelDock = self.p_create_dock("ModelDock",
                                            self.tr("模型选择"), widget)
        ModelRegion = QtWidgets.QVBoxLayout()
        ModelRegion.setObjectName("ModelRegion")
        # labShowSet = self.create_text(CentralWidget, "labShowSet", "模型选择")
        # ModelRegion.addWidget(labShowSet)
        # combo = QtWidgets.QComboBox(self)
        # # combo.addItems([self.tr(ModelsNick[m.__name__][0]) for m in MODELS])
        # combo.addItems([self.tr(ModelsNick[m][0]) for m in ModelsNick.keys()])
        # self.comboModelSelect = combo
        # ModelRegion.addWidget(self.comboModelSelect)
        # 网络参数
        self.btnParamsSelect = self.p_create_button(
            "btnParamsLoad",
            self.tr("加载网络参数"),
            osp.join(pjpath, "resource/Model.png"),
            "Ctrl+D", )
        ModelRegion.addWidget(self.btnParamsSelect)  # 模型选择
        self.cheWithMask = QtWidgets.QCheckBox(self)
        self.cheWithMask.setText(self.tr("使用掩膜"))
        self.cheWithMask.setChecked(True)
        ModelRegion.addWidget(self.cheWithMask)  # with_mask
        horizontalLayout.addLayout(ModelRegion)
        MainWindow.addDockWidget(QtCore.Qt.RightDockWidgetArea, self.ModelDock)
        # 数据列表
        # TODO: 数据列表加一个搜索功能
        widget = QtWidgets.QWidget()
        horizontalLayout = QtWidgets.QHBoxLayout(widget)
        ListRegion = QtWidgets.QVBoxLayout()
        ListRegion.setObjectName("ListRegion")
        # labFiles = self.create_text(CentralWidget, "labFiles", "数据列表")
        # ListRegion.addWidget(labFiles)
        self.listFiles = QtWidgets.QListWidget(self.CentralWidget)
        self.listFiles.setObjectName("ListFiles")
        ListRegion.addWidget(self.listFiles)

        # ListRegion.addWidget(self.btnSave)
        horizontalLayout.addLayout(ListRegion)
        self.DataDock = self.p_create_dock("DataDock", self.tr("数据列表"), widget)
        MainWindow.addDockWidget(QtCore.Qt.RightDockWidgetArea, self.DataDock)
        # 标签列表
        widget = QtWidgets.QWidget()
        horizontalLayout = QtWidgets.QHBoxLayout(widget)
        LabelRegion = QtWidgets.QVBoxLayout()
        LabelRegion.setObjectName("LabelRegion")
        self.labelListTable = TableWidget(
            self.CentralWidget)  # QtWidgets.QTableWidget(CentralWidget)
        self.labelListTable.horizontalHeader().hide()
        # 铺满
        self.labelListTable.horizontalHeader().setSectionResizeMode(
            QtWidgets.QHeaderView.Stretch)
        self.labelListTable.verticalHeader().hide()
        self.labelListTable.setColumnWidth(0, 10)
        # self.labelListTable.setMinimumWidth()
        self.labelListTable.setObjectName("labelListTable")
        self.labelListTable.clearContents()
        self.labelListTable.setRowCount(0)
        self.labelListTable.setColumnCount(4)

        LabelRegion.addWidget(self.labelListTable)
        self.btnAddClass = self.p_create_button(
            "btnAddClass",
            self.tr("添加标签"), osp.join(pjpath, "resource/Label.png"))
        LabelRegion.addWidget(self.btnAddClass)
        horizontalLayout.addLayout(LabelRegion)
        self.LabelDock = self.p_create_dock("LabelDock",
                                            self.tr("标签列表"), widget)
        MainWindow.addDockWidget(QtCore.Qt.RightDockWidgetArea, self.LabelDock)
        ## 滑块设置
        # 分割阈值
        self.p_create_slider = partial(self.create_slider, self.CentralWidget)
        widget = QtWidgets.QWidget()
        horizontalLayout = QtWidgets.QHBoxLayout(widget)
        ShowSetRegion = QtWidgets.QVBoxLayout()
        ShowSetRegion.setObjectName("ShowSetRegion")
        self.sldThresh, _, SegShowRegion = self.p_create_slider(
            "sldThresh", "labThresh", self.tr("分割阈值："))
        ShowSetRegion.addLayout(SegShowRegion)
        ShowSetRegion.addWidget(self.sldThresh)
        # 透明度
        self.sldOpacity, _, MaskShowRegion = self.p_create_slider(
            "sldOpacity", "labOpacity", self.tr("标签透明度："), 75)
        ShowSetRegion.addLayout(MaskShowRegion)
        ShowSetRegion.addWidget(self.sldOpacity)
        # 点大小
        self.sldClickRadius, _, PointShowRegion = self.p_create_slider(
            "sldClickRadius", "labClickRadius",
            self.tr("点击可视化半径："), 3, 10, 0, 1)
        ShowSetRegion.addLayout(PointShowRegion)
        ShowSetRegion.addWidget(self.sldClickRadius)
        # 保存
        self.btnSave = self.p_create_button(
            "btnSave",
            self.tr("保存"),
            osp.join(pjpath, "resource/Save.png"),
            "Ctrl+S", )
        ShowSetRegion.addWidget(self.btnSave)
        horizontalLayout.addLayout(ShowSetRegion)
        self.SegSettingDock = self.p_create_dock("SegSettingDock",
                                                 self.tr("分割设置"), widget)
        MainWindow.addDockWidget(QtCore.Qt.RightDockWidgetArea,
                                 self.SegSettingDock)
        ## 专业功能区工作区
        ## 遥感影像设置
        widget = QtWidgets.QWidget()
        horizontalLayout = QtWidgets.QHBoxLayout(widget)
        self.RSDock = self.p_create_dock("RSDock", self.tr("遥感设置"), widget)
        bandRegion = QtWidgets.QVBoxLayout()
        bandRegion.setObjectName("bandRegion")
        bandRegion.addWidget(
            create_text(self.CentralWidget, "bandSelection", self.tr("波段设置")))
        text_list = ["R", "G", "B"]
        self.bandCombos = []
        for txt in text_list:
            combo = QtWidgets.QComboBox()
            combo.addItems(["band_1"])
            self.bandCombos.append(combo)
            hbandLayout = QtWidgets.QHBoxLayout()
            hbandLayout.setObjectName("hbandLayout")
            hbandLayout.addWidget(
                create_text(self.CentralWidget, "band" + txt, txt))
            hbandLayout.addWidget(combo)
            hbandLayout.setStretch(1, 4)
            bandRegion.addLayout(hbandLayout)
        bandRegion.addWidget(
            create_text(self.CentralWidget, "resultSave", self.tr("保存设置")))
        self.boundaryRegular = QtWidgets.QCheckBox(self.tr("建筑边界规范化"))
        self.boundaryRegular.setObjectName("boundaryRegular")
        bandRegion.addWidget(self.boundaryRegular)
        self.shpSave = QtWidgets.QCheckBox(self.tr("另存为shapefile"))
        self.shpSave.setObjectName("shpSave")
        bandRegion.addWidget(self.shpSave)
        horizontalLayout.addLayout(bandRegion)
        bandRegion.addWidget(
            create_text(self.CentralWidget, "showGeoInfo", self.tr("地理信息")))
        self.edtGeoinfo = QtWidgets.QTextEdit(self.tr("无"))
        self.edtGeoinfo.setObjectName("edtGeoinfo")
        self.edtGeoinfo.setReadOnly(True)
        bandRegion.addWidget(self.edtGeoinfo)
        MainWindow.addDockWidget(QtCore.Qt.LeftDockWidgetArea, self.RSDock)
        ## 医学影像设置
        widget = QtWidgets.QWidget()
        horizontalLayout = QtWidgets.QHBoxLayout(widget)
        MIRegion = QtWidgets.QVBoxLayout()
        MIRegion.setObjectName("MIRegion")
        # mi_text = create_text(CentralWidget, "sliceSelection", self.tr("切片选择"))
        # MIRegion.addWidget(mi_text)
        # self.sldMISlide, _, slideRegion = p_create_slider(
        #     "sldMISlide", "labMISlide", self.tr("切片选择："), 1, 1, 1
        # )
        # self.sldMISlide.setMinimum(1)
        # MIRegion.addLayout(slideRegion)
        # MIRegion.addWidget(self.sldMISlide)
        self.sldWw, self.textWw, WwRegion = self.p_create_slider(
            "sldWw", "textWw", self.tr("窗宽："), 200, 2048, -2048, 1, True)
        MIRegion.addLayout(WwRegion)
        MIRegion.addWidget(self.sldWw)
        self.sldWc, self.textWc, WcRegion = self.p_create_slider(
            "sldWc", "textWc", self.tr("窗位："), 0, 2048, -2048, 1, True)
        MIRegion.addLayout(WcRegion)
        MIRegion.addWidget(self.sldWc)
        horizontalLayout.addLayout(MIRegion)
        self.MedDock = self.p_create_dock("MedDock", self.tr("医疗设置"), widget)
        MainWindow.addDockWidget(QtCore.Qt.LeftDockWidgetArea, self.MedDock)
        ## 宫格区域设置
        widget = QtWidgets.QWidget()
        horizontalLayout = QtWidgets.QHBoxLayout(widget)
        self.GridDock = self.p_create_dock("GridDock", self.tr("宫格切换"), widget)
        GridRegion = QtWidgets.QVBoxLayout()
        GridRegion.setObjectName("GridRegion")
        # self.btnInitGrid = p_create_button(
        #     "btnInitGrid",
        #     self.tr("创建宫格"),
        #     osp.join(pjpath, "resource/N2.png"),
        #     "",
        # )
        self.btnFinishedGrid = self.p_create_button(
            "btnFinishedGrid",
            self.tr("完成宫格"),
            osp.join(pjpath, "resource/Save.png"),
            "", )
        hbandLayout = QtWidgets.QHBoxLayout()
        hbandLayout.setObjectName("hbandLayout")
        # hbandLayout.addWidget(self.btnInitGrid)
        hbandLayout.addWidget(self.btnFinishedGrid)
        GridRegion.addLayout(hbandLayout)  # 创建宫格
        self.cheSaveEvery = QtWidgets.QCheckBox(self)
        self.cheSaveEvery.setText(self.tr("保存每个宫格的标签"))
        self.cheSaveEvery.setChecked(False)
        GridRegion.addWidget(self.cheSaveEvery)
        self.gridTable = QtWidgets.QTableWidget(self.CentralWidget)
        self.gridTable.horizontalHeader().hide()
        self.gridTable.verticalHeader().hide()
        # 铺满
        self.gridTable.horizontalHeader().setSectionResizeMode(
            QtWidgets.QHeaderView.Stretch)
        self.gridTable.verticalHeader().setSectionResizeMode(
            QtWidgets.QHeaderView.Stretch)
        self.gridTable.setObjectName("gridTable")
        self.gridTable.clearContents()
        self.gridTable.setColumnCount(1)
        self.gridTable.setRowCount(1)
        GridRegion.addWidget(self.gridTable)
        horizontalLayout.addLayout(GridRegion)
        MainWindow.addDockWidget(QtCore.Qt.LeftDockWidgetArea, self.GridDock)
        ## 视频设置
        widget = QtWidgets.QWidget()
        horizontalLayout = QtWidgets.QHBoxLayout(widget)
        self.VSTDock = self.p_create_dock("VSTDock", self.tr("视频分割设置"), widget)
        VSTDock = QtWidgets.QVBoxLayout()
        VSTDock.setObjectName("VSTDock")
        self.btn3DParamsSelect = self.p_create_button(
            "btn3DParamsLoad",
            self.tr("加载传播网络参数"),
            osp.join(pjpath, "resource/Model.png"),
            "", )
        VSTDock.addWidget(self.btn3DParamsSelect)
        self.btnPropagate = self.p_create_button(
            "btnPropagate",
            self.tr("传播"),
            osp.join(pjpath, "resource/Propagate.png"),
            "", )
        self.proPropagete = QtWidgets.QProgressBar(self.CentralWidget)
        self.proPropagete.setAlignment(QtCore.Qt.AlignVCenter)
        self.proPropagete.setValue(0)
        VSTDock.addWidget(self.btnPropagate)
        proLayer = QtWidgets.QHBoxLayout()
        proLayer.addWidget(
            create_text(self.CentralWidget, None, self.tr("进度：")))
        proLayer.addWidget(self.proPropagete)
        VSTDock.addLayout(proLayer)
        horizontalLayout.addLayout(VSTDock)
        MainWindow.addDockWidget(QtCore.Qt.LeftDockWidgetArea, self.VSTDock)
        # 3d显示
        widget = QtWidgets.QWidget()
        horizontalLayout = QtWidgets.QHBoxLayout(widget)
        self.TDDock = self.p_create_dock("TDDock", self.tr("3D显示"), widget)
        TDRegion = QtWidgets.QVBoxLayout()
        TDRegion.setObjectName("TDRegion")
        self.vtkWidget = VTKWidget(self.CentralWidget)
        TDRegion.addWidget(self.vtkWidget)
        horizontalLayout.addLayout(TDRegion)
        MainWindow.addDockWidget(QtCore.Qt.LeftDockWidgetArea, self.TDDock)
        # 视频播放功能区域
        widget = QtWidgets.QWidget()
        verticalLayout = QtWidgets.QVBoxLayout(widget)
        self.VideoDock = self.p_create_dock("VideoDock", self.tr("时间轴"), widget)
        VideoRegion = QtWidgets.QHBoxLayout()
        VideoRegion.setObjectName("VideoRegion")
        self.videoPlay = self.p_create_button(
            "videoPlay",
            self.tr("播放"),
            osp.join(pjpath, "resource/Play.png"),
            "", )
        height = self.videoPlay.height()
        min_size = QtCore.QSize(0, height)
        VideoRegion.addWidget(self.videoPlay)
        self.sldTime, self.textTime, _ = self.p_create_slider(
            "sldTime", "textTime", "", 0, 100, 0, 1, True)
        self.textTime.setMinimumSize(min_size)
        self.sldTime.setTickInterval(1)
        self.sldTime.setTickPosition(QtWidgets.QSlider.TicksBelow)
        VideoRegion.addWidget(self.sldTime)
        VideoRegion.addWidget(
            create_text(self.CentralWidget, None, self.tr("帧数：")))
        VideoRegion.addWidget(self.textTime)
        self.preFrameButton = self.p_create_button(
            "preFrameButton",
            self.tr("上一帧"), )
        self.nextFrameButton = self.p_create_button(
            "nextFrameButton",
            self.tr("下一帧"), )
        # self.preFrameButton = QtWidgets.QPushButton("上一帧")
        # self.nextFrameButton = QtWidgets.QPushButton("下一帧")
        VideoRegion.addWidget(self.preFrameButton)
        VideoRegion.addWidget(self.nextFrameButton)
        self.speedComboBox = QtWidgets.QComboBox()
        self.speedComboBox.setMinimumSize(min_size)
        v_txt = self.tr("倍速: ")
        self.speedComboBox.addItems([
            v_txt + "0.25x",
            v_txt + "0.5x",
            v_txt + "1.0x",
            v_txt + "2.0x",
            v_txt + "3.0x",
            v_txt + "5.0x",
            v_txt + "10.0x",
        ])
        self.speedComboBox.setCurrentIndex(2)
        VideoRegion.addWidget(self.speedComboBox)
        verticalLayout.addLayout(VideoRegion)
        self.VideoDock.setAllowedAreas(QtCore.Qt.BottomDockWidgetArea)
        MainWindow.addDockWidget(QtCore.Qt.BottomDockWidgetArea, self.VideoDock)

        # timer
        self.timer = QTimer()
        self.timer.setSingleShot(False)

        ## -----
        QtCore.QMetaObject.connectSlotsByName(MainWindow)
        # log.debug("Set up UI finished")

    ## 创建文本

    def create_text(self, parent, text_name=None, text_text=None):
        return create_text(parent, text_name, text_text)

    ## 创建按钮

    def create_button(self,
                      parent,
                      btn_name,
                      btn_text,
                      ico_path=None,
                      curt=None):
        return create_button(parent, btn_name, btn_text, ico_path, curt)

    ## 创建dock
    def creat_dock(self, parent, name, text, layout):
        return creat_dock(parent, name, text, layout)

    ## 显示Logo
    def show_logo(self, logo_path):
        labLogo = QtWidgets.QLabel()
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum,
                                           QtWidgets.QSizePolicy.Maximum)
        labLogo.setSizePolicy(sizePolicy)
        labLogo.setMaximumSize(QtCore.QSize(100, 33))
        labLogo.setPixmap(QtGui.QPixmap(logo_path))
        labLogo.setScaledContents(True)
        labLogo.setObjectName("labLogo")
        return labLogo

    ## 创建滑块区域
    def create_slider(self,
                      parent,
                      sld_name,
                      text_name,
                      text,
                      default_value=50,
                      max_value=100,
                      min_value=0,
                      text_rate=0.01,
                      edit=False):
        return create_slider(parent, sld_name, text_name, text, default_value,
                             max_value, min_value, text_rate, edit)



# Copyright (c) 2021 PaddlePaddle Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import numpy as np
import cv2

from eiseg import logger


def has_sitk():
    try:
        import SimpleITK

        return True
    except ImportError:
        return False


if has_sitk():
    import SimpleITK as sitk


def dcm_reader(path):
    logger.debug(f"opening medical image {path}")
    reader = sitk.ImageSeriesReader()
    reader.SetFileNames([path])
    image = reader.Execute()
    img = sitk.GetArrayFromImage(image)
    logger.debug(f"scan shape is {img.shape}")
    if len(img.shape) == 4:
        img = img[0]
    # WHC
    img = np.transpose(img, [1, 2, 0])
    return img.astype(np.int32)


def windowlize(scan, ww, wc):
    wl = wc - ww / 2
    wh = wc + ww / 2
    res = scan.copy()
    res = res.astype(np.float32)
    res = np.clip(res, wl, wh)
    res = (res - wl) / ww * 255
    res = res.astype(np.uint8)
    # print("++", res.shape)
    # for idx in range(res.shape[-1]):
    # TODO: 支持3d或者改调用
    res = cv2.cvtColor(res, cv2.COLOR_GRAY2BGR)

    return res


# def open_nii(niiimg_path):
#     if IPT_SITK == True:
#         sitk_image = sitk.ReadImage(niiimg_path)
#         return _nii2arr(sitk_image)
#     else:
#         raise ImportError("can't import SimpleITK!")

#
# def _nii2arr(sitk_image):
#     if IPT_SITK == True:
#         img = sitk.GetArrayFromImage(sitk_image).transpose((1, 2, 0))
#         return img
#     else:
#         raise ImportError("can't import SimpleITK!")
#
#
# def slice_img(img, index):
#     if index == 0:
#         return sample_norm(
#             cv2.merge(
#                 [
#                     np.uint16(img[:, :, index]),
#                     np.uint16(img[:, :, index]),
#                     np.uint16(img[:, :, index + 1]),
#                 ]
#             )
#         )
#     elif index == img.shape[2] - 1:
#         return sample_norm(
#             cv2.merge(
#                 [
#                     np.uint16(img[:, :, index - 1]),
#                     np.uint16(img[:, :, index]),
#                     np.uint16(img[:, :, index]),
#                 ]
#             )
#         )
#     else:
#         return sample_norm(
#             cv2.merge(
#                 [
#                     np.uint16(img[:, :, index - 1]),
#                     np.uint16(img[:, :, index]),
#                     np.uint16(img[:, :, index + 1]),
#                 ]
#             )
#         )



from functools import wraps
from copy import deepcopy
import inspect
import paddle.nn as nn


def serialize(init):
    parameters = list(inspect.signature(init).parameters)

    @wraps(init)
    def new_init(self, *args, **kwargs):
        params = deepcopy(kwargs)
        for pname, value in zip(parameters[1:], args):
            params[pname] = value

        config = {"class": get_classname(self.__class__), "params": dict()}
        specified_params = set(params.keys())

        for pname, param in get_default_params(self.__class__).items():
            if pname not in params:
                params[pname] = param.default

        for name, value in list(params.items()):
            param_type = "builtin"
            if inspect.isclass(value):
                param_type = "class"
                value = get_classname(value)

            config["params"][name] = {
                "type": param_type,
                "value": value,
                "specified": name in specified_params,
            }

        setattr(self, "_config", config)
        init(self, *args, **kwargs)

    return new_init


def load_model(config, **kwargs):
    model_class = get_class_from_str(config["class"])
    model_default_params = get_default_params(model_class)

    model_args = dict()
    for pname, param in config["params"].items():
        value = param["value"]
        if param["type"] == "class":
            value = get_class_from_str(value)

        if pname not in model_default_params and not param["specified"]:
            continue

        assert pname in model_default_params
        if not param["specified"] and model_default_params[
                pname].default == value:
            continue
        model_args[pname] = value

    model_args.update(kwargs)

    return model_class(**model_args)


def get_config_repr(config):
    config_str = f'Model: {config["class"]}\n'
    for pname, param in config["params"].items():
        value = param["value"]
        if param["type"] == "class":
            value = value.split(".")[-1]
        param_str = f"{pname:<22} = {str(value):<12}"
        if not param["specified"]:
            param_str += " (default)"
        config_str += param_str + "\n"
    return config_str


def get_default_params(some_class):
    params = dict()
    for mclass in some_class.mro():
        if mclass is nn.Layer or mclass is object:
            continue

        mclass_params = inspect.signature(mclass.__init__).parameters
        for pname, param in mclass_params.items():
            if param.default != param.empty and pname not in params:
                params[pname] = param

    return params


def get_classname(cls):
    module = cls.__module__
    name = cls.__qualname__
    if module is not None and module != "__builtin__":
        name = module + "." + name
    return name


def get_class_from_str(class_str):
    components = class_str.split(".")
    mod = __import__(".".join(components[:-1]))
    for comp in components[1:]:
        mod = getattr(mod, comp)
    return mod



import numpy as np


def get_color_map(N=256):
    def bitget(byteval, idx):
        return ((byteval & (1 << idx)) != 0)

    cmap = np.zeros((N, 3), dtype=np.uint8)
    for i in range(N):
        r = g = b = 0
        c = i
        for j in range(8):
            r = r | (bitget(c, 0) << 7 - j)
            g = g | (bitget(c, 1) << 7 - j)
            b = b | (bitget(c, 2) << 7 - j)
            c = c >> 3

        cmap[i] = np.array([r, g, b])

    return cmap


color_map = get_color_map()


def pal_color_map():
    return color_map



import inspect
from collections.abc import Sequence


class ComponentManager:
    def __init__(self, name=None):
        self._components_dict = dict()
        self._name = name

    def __len__(self):
        return len(self._components_dict)

    def __repr__(self):
        name_str = self._name if self._name else self.__class__.__name__
        return "{}:{}".format(name_str, list(self._components_dict.keys()))

    def __getitem__(self, item):
        if isinstance(item, int):
            if item >= len(self):
                raise KeyError(f"指定的下标 {item} 在长度为 {len(self)} 的 {self} 中越界")
            return list(self._components_dict.values())[item]
        if item not in self._components_dict.keys():
            raise KeyError(f"{self} 中不存在 {item}")
        return self._components_dict[item]

    def __iter__(self):
        for val in self._components_dict.values():
            yield val

    def keys(self):
        return list(self._components_dict.keys())

    def idx(self, item):
        for idx, val in enumerate(self.keys()):
            if val == item:
                return idx
        raise KeyError(f"{item} is not in {self}")

    @property
    def components_dict(self):
        return self._components_dict

    @property
    def name(self):
        return self._name

    def _add_single_component(self, component):
        # Currently only support class or function type
        if not (inspect.isclass(component) or inspect.isfunction(component)):
            raise TypeError("Expect class/function type, but received {}".
                            format(type(component)))

        # Obtain the internal name of the component
        component_name = component.__name__

        # Check whether the component was added already
        if component_name in self._components_dict.keys():
            raise KeyError("{} exists already!".format(component_name))
        else:
            # Take the internal name of the component as its key
            self._components_dict[component_name] = component

    def add_component(self, components):
        # Check whether the type is a sequence
        if isinstance(components, Sequence):
            for component in components:
                self._add_single_component(component)
        else:
            component = components
            self._add_single_component(component)

        return components


MODELS = ComponentManager("models")
ACTIONS = ComponentManager("actions")



# Copyright (c) 2021 PaddlePaddle Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from qtpy import QtWidgets, QtGui, QtCore
from qtpy.QtCore import Qt


# Note, bbox annotation is more convenient than the default boundingBox generated by QGrpaphicItem
class BBoxAnnotation(QtWidgets.QGraphicsPathItem):
    def __init__(
            self,
            labelIndex,
            polyline,
            borderColor=[0, 0, 255],
            cocoIndex=None,
            parent=None, ):
        super(BBoxAnnotation, self).__init__(parent)
        self.polyline = polyline
        self.corner_points = []
        self.upper_right = QtCore.QPointF()
        self.bottom_left = QtCore.QPointF()
        self.w = -1.0
        self.h = -1.0

        self.parent = parent
        self.is_added = False
        if self.parent is not None:
            self.is_added = True
        self.labelIndex = labelIndex
        self.coco_id = cocoIndex
        self.bbox_hovering = True

        # set rendering attributes
        self.setZValue(10)

        # b = borderColor
        # self.borderColor = QtGui.QColor(b[0], b[1], b[2])
        self.borderColor = QtGui.QColor(128, 128, 128)
        self.borderColor.setAlphaF(0.8)
        pen = QtGui.QPen(self.borderColor, 1.2)
        pen.setStyle(Qt.DashDotLine)
        self.setPen(pen)

        self.setFlag(QtWidgets.QGraphicsItem.ItemIsSelectable, False)
        self.setFlag(QtWidgets.QGraphicsItem.ItemIsMovable, False)
        self.setFlag(QtWidgets.QGraphicsItem.ItemSendsGeometryChanges, True)
        self.setFlag(QtWidgets.QGraphicsItem.ItemIsFocusable, False)
        self.setAcceptHoverEvents(False)
        # self.setCursor(QtGui.QCursor(QtCore.Qt.PointingHandCursor))

    @property
    def scnenePoints(self):
        # return 4 corner points
        raise Exception("Not Implemented Yet!")

    def setAnning(self, isAnning=True):
        raise Exception("Not Implemented Yet!")

    def remove(self):
        raise Exception("Not Implemented Yet!")

    # ===== generate geometry info

    def create_corners(self):
        bbox_rect_geo = self.polyline.boundingRect()
        self.bottom_left = bbox_rect_geo.bottomLeft()
        self.upper_right = bbox_rect_geo.topRight()
        self.corner_points.clear()
        self.corner_points.extend([
            self.bottom_left,
            bbox_rect_geo.topLeft(),
            self.upper_right,
            bbox_rect_geo.bottomRight(),
        ])
        self.w = self.corner_points[3].x() - self.corner_points[1].x()
        self.h = self.corner_points[3].y() - self.corner_points[1].y()

        if self.corner_points[1].x() > 512 or self.corner_points[1].x(
        ) + self.w > 512:
            pass
        if self.corner_points[1].y() > 512 or self.corner_points[1].y(
        ) + self.h > 512:
            pass
        return self.corner_points

    def create_lines(self):
        pass

    # ===== graphic interface to update in scene tree

    def update(self):
        l = len(self.polyline.points)
        # print("up L:", l, " is_added:", self.is_added)
        if l < 3:
            if self.is_added:
                self.remove_from_scene()
        else:  # 大于三个点就可以更新，小于三个点删除多边形
            if self.is_added:
                self.add_to_scene()
            else:
                path_geo = QtGui.QPainterPath()
                self.create_corners()
                path_geo.moveTo(self.corner_points[0])
                for i in range(4):
                    path_geo.lineTo(self.corner_points[(i + 1) % 4])
                self.setPath(QtGui.QPainterPath(path_geo))
                pass
            pass
        pass

    def add_to_scene(self):
        # self.parentItem().scene().addItem(self)
        self.setParentItem(self.parent)
        self.is_added = True

    def remove_from_scene(self):
        # self.parentItem().scene().removeItem(self)
        self.setParentItem(None)
        self.is_added = False

    # ===== annotation info

    # @return : [x, y, w, h]
    def to_array(self):
        np_array = [
            self._round(self.corner_points[1].x()),
            self._round(self.corner_points[1].y()),  # topLeft
            self._round(self.w),
            self._round(self.h),
        ]
        return np_array

    def _round(self, number, ind=0):
        nint, ndec = str(number).split(".")
        res = float(nint + "." + ndec[:ind])
        if res <= 0:
            res = 0.0
        return res

    def __del__(self):
        self.corner_points.clear()



# Copyright (c) 2021 PaddlePaddle Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from PyQt5.QtCore import QPointF
from qtpy import QtWidgets, QtGui, QtCore


class GripItem(QtWidgets.QGraphicsPathItem):
    maxSize = 1.5
    minSize = 0.8

    def __init__(self, annotation_item, index, color, img_size):
        super(GripItem, self).__init__()
        self.m_annotation_item = annotation_item
        self.hovering = False
        self.m_index = index
        self.anning = True
        color.setAlphaF(1)
        self.color = color
        self.img_size = img_size

        self.updateSize()
        self.setPath(self.circle)
        self.setBrush(self.color)
        self.setPen(QtGui.QPen(self.color, 1))
        self.setFlag(QtWidgets.QGraphicsItem.ItemIsSelectable, True)
        self.setFlag(QtWidgets.QGraphicsItem.ItemIsMovable, True)
        self.setFlag(QtWidgets.QGraphicsItem.ItemSendsGeometryChanges, True)
        self.setFlag(QtWidgets.QGraphicsItem.ItemIsFocusable, True)
        self.setAcceptHoverEvents(True)
        self.setZValue(12)
        self.setCursor(QtGui.QCursor(QtCore.Qt.PointingHandCursor))

    def setColor(self, color):
        self.setBrush(color)
        self.setPen(QtGui.QPen(color, 1))
        self.color = color

    def setAnning(self, anning=True):
        self.anning = anning
        self.setEnabled(anning)

    # BUG: Scaling causes a crash
    @property
    def size(self):
        return GripItem.minSize
        # if not self.scene():
        #     return GripItem.minSize
        # else:
        #     maxi, mini = GripItem.maxSize, GripItem.minSize
        #     exp = 1 - mini / maxi
        #     size = maxi * (1 - exp ** self.scene().scale)
        #     if size > GripItem.maxSize:
        #         size = GripItem.maxSize
        #     if size < GripItem.minSize:
        #         size = GripItem.minSize
        #     return size

    def updateSize(self, s=2):
        size = self.size
        self.circle = QtGui.QPainterPath()
        self.circle.addEllipse(QtCore.QRectF(-size, -size, size * s, size * s))
        self.square = QtGui.QPainterPath()
        self.square.addRect(QtCore.QRectF(-size, -size, size * s, size * s))
        self.setPath(self.square if self.hovering else self.circle)

    def hoverEnterEvent(self, ev):
        self.setPath(self.square)
        self.setBrush(QtGui.QColor(0, 0, 0, 0))
        self.m_annotation_item.item_hovering = True
        self.hovring = True
        super(GripItem, self).hoverEnterEvent(ev)

    def hoverLeaveEvent(self, ev):
        self.setPath(self.circle)
        self.setBrush(self.color)
        self.m_annotation_item.item_hovering = False
        self.hovring = False
        super(GripItem, self).hoverLeaveEvent(ev)

    def mouseReleaseEvent(self, ev):
        self.setSelected(False)
        super(GripItem, self).mouseReleaseEvent(ev)

    def itemChange(self, change, value):
        tmp_val = value
        if change == QtWidgets.QGraphicsItem.ItemPositionChange and self.isEnabled(
        ):
            if value.x() > self.img_size[1]:
                x = self.img_size[1]
            elif value.x() < 0:
                x = 0
            else:
                x = value.x()
            if value.y() > self.img_size[0]:
                y = self.img_size[0]
            elif value.y() < 0:
                y = 0
            else:
                y = value.y()
            tmp_val = QPointF(x, y)
            self.m_annotation_item.movePoint(self.m_index, tmp_val)
            self.m_annotation_item.setDirty(True)
        return super(GripItem, self).itemChange(change, tmp_val)

    def shape(self):
        path = QtGui.QPainterPath()
        p = self.mapFromScene(self.pos())
        x, y = p.x(), p.y()
        s = self.size
        path.addEllipse(p, s + GripItem.minSize, s + GripItem.minSize)
        return path

    def mouseDoubleClickEvent(self, ev):
        self.m_annotation_item.removeFocusPoint()
